#include <math.h>
#include <string.h>
#include <iostream>
#include <fstream>
using namespace std;

const int MAXS = 100;
const int MAXP = 200;

enum Type {CAFE, RESTAURANT, STATION, HOTEL, MUSEUM};

struct Coord {
	double x;
	double y;
};

struct POI {
	Coord coord;
	Type type;
	char name[MAXS];
	int stars;
};

struct Map {
	POI pois[MAXP];
	int numPois;
};

double eucDistance(const Coord a, const Coord b){
	return sqrt(pow(a.x-b.x, 2) + pow(a.y-b.y, 2));
}

bool findCloser(const Map* pm, const Coord c, POI* pp) {
	if(pm->numPois == 0) 
		return false;
	double mind = eucDistance(pm->pois[0].coord, c);
	int index = 0;	
	for(int i=1; i<pm->numPois; i++) {
		double dd = eucDistance(pm->pois[i].coord, c);
		if(dd < mind) {
			mind = dd;
			index = i;
		}
	}
	*pp = pm->pois[index];
	return true;
}

void sort(Map* pm) {
	for(int i=0; i<pm->numPois-1; i++) {
		int index = i;
		for(int j=i+1;j<pm->numPois; j++) {
			if(pm->pois[j].type<pm->pois[index].type)
				index = j;
		}
		POI tmp = pm->pois[i];
		pm->pois[i] = pm->pois[index];
		pm->pois[index] = tmp;
	}
}

bool saveByCategory(const Map* pm, const char fn[]) {
	Map aux;
	aux = *pm;
	sort(&aux);
  fstream ff;
  ff.open(fn, ios::out);
  if(!ff)
    return false;
	for(int i=0; i<aux.numPois; i++) {
		switch(aux.pois[i].type) {
			case CAFE:
				ff << "Cafe ";
				break;
			case RESTAURANT:
				ff << "Restaurant ";
				break;
			case STATION:
				ff << "Station ";
				break;
			case HOTEL:
				ff << "Hotel ";
				break;
			case MUSEUM:
				ff << "Museum ";
				break;
		}
		ff << aux.pois[i].name << '\t' << aux.pois[i].coord.x << ", " 
		<< aux.pois[i].coord.y << ' ' << aux.pois[i].stars << endl;
	}
  return true;
}

double manhattan(Coord a, Coord b) {
	return fabs(a.x-b.x) + fabs(a.y-b.y);
}

bool distances(const Map* pm, const char name1[], const char name2[], double* euc, double* man) {
	bool found1 = false, found2 = false;
	Coord c1, c2;
	for(int i=0; i<pm->numPois; i++) {
		if(strcmp(pm->pois[i].name, name1) == 0) {
			c1 = pm->pois[i].coord;
			found1 = true;
		}
		if(strcmp(pm->pois[i].name, name2) == 0) {
			c2 = pm->pois[i].coord;
			found2 = true;
		}
	}
	if(found1 && found2) {
		*euc = eucDistance(c1, c2);
		*man = manhattan(c1, c2);
		return true;
	}
	return false;
}

int remove(Map* pm, int s) {
	int count = 0;
	for(int i=0; i<pm->numPois; i++) {
		if(pm->pois[i].stars <= s) {
			pm->pois[i] = pm->pois[--pm->numPois];
			count++;
			i--;
		}
	}
	return count;
}

bool contained(Coord a, Coord b, Coord c) {
	return a.x>=b.x && a.x < c.x && a.y >= b.y && a.y < c.y;
}

bool present(const Map* pm, Coord c1, Coord c2) {
	for(int i=0; i<pm->numPois; i++)
		if(contained(pm->pois[i].coord, c1, c2)) return true;
	return false;
}

void draw(const Map* pm) {
	for(int i=0; i<10; i++) {
		for(int j=0; j<10; j++){
			Coord ul = {10*j, 10*i};
			Coord dr = {10*(j+1), 10*(i+1)};
			if(present(pm,ul, dr)) cout << 'X';
			else cout << '.';
		}
		cout << endl;
	}
}
